@@ -1,38 +1,38 @@
-import operator
 import itertools
+import logging
+import operator
 
+import astropy.units as u
 import numpy as np
+from astropy.modeling import fitting, Model, models
+from astropy.table import QTable
 from scipy.signal import convolve
+
+
 import astropy.units as u
 from astropy.stats import sigma_clipped_stats
 
-from ..manipulation.utils import excise_regions
-from ..analysis import fwhm, centroid
-from ..utils import QuantityModel
-from ..manipulation import extract_region, noise_region_uncertainty
 from ..spectra.spectral_region import SpectralRegion
 from ..spectra.spectrum1d import Spectrum1D
-from astropy.modeling import fitting, Model, models
-from astropy.table import QTable
-
+from ..utils import QuantityModel
+from ..analysis import fwhm, gaussian_sigma_width, centroid, warn_continuum_below_threshold
+from ..manipulation import extract_region, noise_region_uncertainty
+from ..manipulation.utils import excise_regions
 
 __all__ = ['find_lines_threshold', 'find_lines_derivative', 'fit_lines',
            'estimate_line_parameters']
 
-# Define the initial estimators
-#   This is the default methods to use to estimate astropy model
-#   parameters. This is based on only a small subset of the astropy
-#   models but it was determined that this is a decent start as most
-#   fitting will probably use one of these.
+# Define the initial estimators. This are the default methods to use to
+# estimate astropy model parameters. This is based on only a small subset of
+# the astropy models but it was determined that this is a decent start as most
+# fitting will probably use one of these.
 #
-#   Each method list must take a Spectrum1D object and should return
-#   a Quantity.
-
+# Each method list must take a Spectrum1D object and should return a Quantity.
 _parameter_estimators = {
     'Gaussian1D': {
         'amplitude': lambda s: max(s.flux),
         'mean': lambda s: centroid(s, region=None),
-        'stddev': lambda s: fwhm(s)
+        'stddev': lambda s: gaussian_sigma_width(s)
     },
     'Lorentz1D': {
         'amplitude': lambda s: max(s.flux),
@@ -53,14 +53,19 @@ def _set_parameter_estimators(model):
     Helper method used in method below.
     """
     if model.__class__.__name__ in _parameter_estimators:
-        model._constraints['parameter_estimator'] = _parameter_estimators[model.__class__.__name__]
+        model_pars = _parameter_estimators[model.__class__.__name__]
+        for name in model.param_names:
+            par = getattr(model, name)
+            setattr(par, "estimator", model_pars[name])
+
     return model
 
 
 def estimate_line_parameters(spectrum, model):
     """
-    The input ``model`` parameters will be estimated from the input ``spectrum``. The
-    ``model`` can be specified with default parameters, for example ``Gaussian1D()``.
+    The input ``model`` parameters will be estimated from the input
+    ``spectrum``. The ``model`` can be specified with default parameters, for
+    example ``Gaussian1D()``.
 
     Parameters
     ----------
@@ -76,18 +81,16 @@ def estimate_line_parameters(spectrum, model):
         Model with parameters estimated.
     """
 
-    if not 'parameter_estimator' in model._constraints:
-        model = _set_parameter_estimators(model)
-
+    model = _set_parameter_estimators(model)
     # Estimate the parameters based on the estimators already
     # attached to the model
-    if 'parameter_estimator' in model._constraints:
-        for param, estimator in model._constraints['parameter_estimator'].items():
-            setattr(model, param, estimator(spectrum))
-
-    # No estimators.
-    else:
-        raise Exception('No method to estimate parameters')
+    for name in model.param_names:
+        par = getattr(model, name)
+        try:
+            estimator = getattr(par, "estimator")
+            setattr(model, name, estimator(spectrum))
+        except AttributeError:
+            raise Exception('No method to estimate parameter {}'.format(name))
 
     return model
 
@@ -96,15 +99,17 @@ def _consecutive(data, stepsize=1):
     return np.split(data, np.where(np.diff(data) != stepsize)[0]+1)
 
 
+@warn_continuum_below_threshold(threshold=0.01)
 def find_lines_threshold(spectrum, noise_factor=1):
     """
     Find the emission and absorption lines in a spectrum. The method
     here is based on deviations larger than the spectrum's uncertainty by the
     ``noise_factor``.
 
-    This method only works with continuum-subtracted spectra and the uncertainty
-    must be defined on the spectrum. To add the uncertainty, one could use
-    `~specutils.manipulation.noise_region_uncertainty` to add the uncertainty.
+    This method only works with continuum-subtracted spectra and the
+    uncertainty must be defined on the spectrum. To add the uncertainty,
+    one could use `~specutils.manipulation.noise_region_uncertainty` to add
+    the uncertainty.
 
     Parameters
     ----------
@@ -119,18 +124,20 @@ def find_lines_threshold(spectrum, noise_factor=1):
     -------
     qtable: `~astropy.table.QTable`
         Table of emission and absorption lines. Line center (``line_center``),
-        line type (``line_type``) and index of line center (``line_center_index``)
-        are stored for each line.
+        line type (``line_type``) and index of line center
+        (``line_center_index``) are stored for each line.
     """
 
     # Threshold based on noise estimate and factor.
     uncertainty = spectrum.uncertainty
-    inds = np.where(np.abs(spectrum.flux) > (noise_factor*uncertainty.array)*spectrum.flux.unit)[0]
+    inds = np.where(np.abs(spectrum.flux) > (noise_factor*uncertainty.array) *
+                    spectrum.flux.unit)[0]
     pos_inds = inds[spectrum.flux.value[inds] > 0]
     line_inds_grouped = _consecutive(pos_inds, stepsize=1)
 
     if len(line_inds_grouped[0]) > 0:
-        emission_inds = [inds[np.argmax(spectrum.flux.value[inds])] for inds in line_inds_grouped]
+        emission_inds = [inds[np.argmax(spectrum.flux.value[inds])]
+                         for inds in line_inds_grouped]
     else:
         emission_inds = []
 
@@ -142,7 +149,8 @@ def find_lines_threshold(spectrum, noise_factor=1):
     line_inds_grouped = _consecutive(neg_inds, stepsize=1)
 
     if len(line_inds_grouped[0]) > 0:
-        absorption_inds = [inds[np.argmin(spectrum.flux.value[inds])] for inds in line_inds_grouped]
+        absorption_inds = [inds[np.argmin(spectrum.flux.value[inds])]
+                           for inds in line_inds_grouped]
     else:
         absorption_inds = []
 
@@ -151,14 +159,21 @@ def find_lines_threshold(spectrum, noise_factor=1):
     #
 
     qtable = QTable()
-    qtable['line_center'] = list(itertools.chain(*[spectrum.spectral_axis.value[emission_inds],
-                                             spectrum.spectral_axis.value[absorption_inds]]))*spectrum.spectral_axis.unit
-    qtable['line_type'] = ['emission']*len(emission_inds) + ['absorption']*len(absorption_inds)
-    qtable['line_center_index'] = list(itertools.chain(*[emission_inds, absorption_inds]))
+    qtable['line_center'] = list(
+        itertools.chain(
+            *[spectrum.spectral_axis.value[emission_inds],
+              spectrum.spectral_axis.value[absorption_inds]]
+        )) * spectrum.spectral_axis.unit
+    qtable['line_type'] = ['emission'] * len(emission_inds) + \
+                          ['absorption'] * len(absorption_inds)
+    qtable['line_center_index'] = list(
+        itertools.chain(
+            *[emission_inds, absorption_inds]))
 
     return qtable
 
 
+@warn_continuum_below_threshold(threshold=0.01)
 def find_lines_derivative(spectrum, flux_threshold=None):
     """
     Find the emission and absorption lines in a spectrum. The method
@@ -169,20 +184,19 @@ def find_lines_derivative(spectrum, flux_threshold=None):
     ----------
     spectrum : Spectrum1D
         The spectrum object over which the equivalent width will be calculated.
-
     flux_threshold : float, `~astropy.units.Quantity` or None
         The threshold a pixel must be above to be considered part of a line. If
-        a float, will assume the same units as ``spectrum.flux``. This threshold
-        is above and beyond the derivative searching step. Default is None so no
-        thresholding. The threshold is positive for emission lines and negative
-        for absorption lines.
+        a float, will assume the same units as ``spectrum.flux``. This
+        threshold is above and beyond the derivative searching step. Default
+        is None so no thresholding. The threshold is positive for emission
+        lines and negative for absorption lines.
 
     Returns
     -------
     qtable: `~astropy.table.QTable`
         Table of emission and absorption lines. Line center (``line_center``),
-        line type (``line_type``) and index of line center (``line_center_index``)
-        are stored for each line.
+        line type (``line_type``) and index of line center
+        (``line_center_index``) are stored for each line.
     """
 
     # Take the derivative to find the zero crossings which correspond to
@@ -213,7 +227,8 @@ def find_lines_derivative(spectrum, flux_threshold=None):
     line_inds_grouped = _consecutive(line_inds, stepsize=1)
 
     if len(line_inds_grouped[0]) > 0:
-        emission_inds = [inds[np.argmax(spectrum.flux[inds])] for inds in line_inds_grouped]
+        emission_inds = [inds[np.argmax(spectrum.flux[inds])]
+                         for inds in line_inds_grouped]
     else:
         emission_inds = []
 
@@ -232,7 +247,8 @@ def find_lines_derivative(spectrum, flux_threshold=None):
     line_inds_grouped = _consecutive(line_inds, stepsize=1)
 
     if len(line_inds_grouped[0]) > 0:
-        absorption_inds = [inds[np.argmin(spectrum.flux[inds])] for inds in line_inds_grouped]
+        absorption_inds = [inds[np.argmin(spectrum.flux[inds])] for inds in
+                           line_inds_grouped]
     else:
         absorption_inds = []
 
@@ -241,10 +257,16 @@ def find_lines_derivative(spectrum, flux_threshold=None):
     #
 
     qtable = QTable()
-    qtable['line_center'] = list(itertools.chain(*[spectrum.spectral_axis.value[emission_inds],
-                                             spectrum.spectral_axis.value[absorption_inds]]))*spectrum.spectral_axis.unit
-    qtable['line_type'] = ['emission']*len(emission_inds) + ['absorption']*len(absorption_inds)
-    qtable['line_center_index'] = list(itertools.chain(*[emission_inds, absorption_inds]))
+    qtable['line_center'] = list(
+        itertools.chain(
+            *[spectrum.spectral_axis.value[emission_inds],
+              spectrum.spectral_axis.value[absorption_inds]]
+        )) * spectrum.spectral_axis.unit
+    qtable['line_type'] = ['emission'] * len(emission_inds) + \
+                          ['absorption'] * len(absorption_inds)
+    qtable['line_center_index'] = list(
+        itertools.chain(
+            *[emission_inds, absorption_inds]))
 
     return qtable
 
@@ -260,20 +282,21 @@ def fit_lines(spectrum, model, fitter=fitting.LevMarLSQFitter(),
     ----------
     spectrum : Spectrum1D
         The spectrum object over which the equivalent width will be calculated.
-
     model: `~astropy.modeling.Model` or list of `~astropy.modeling.Model`
         The model or list of models that contain the initial guess.
-
+    fitter : `~astropy.modeling.fitting.Fitter`, optional
+        Fitter instance to be used when fitting model to spectrum.
     exclude_regions : list of `~specutils.SpectralRegion`
         List of regions to exclude in the fitting.
-
-    weights : list  (NOT IMPLEMENTED YET)
-        List of weights to define importance of fitting regions.
-
+    weights : array-like or 'unc', optional
+        If 'unc', the unceratinties from the spectrum object are used to
+        to calculate the weights. If array-like, represents the weights to
+        use in the fitting.  Note that if a mask is present on the spectrum, it
+        will be applied to the ``weights`` as it would be to the spectrum
+        itself.
     window : `~specutils.SpectralRegion` or list of `~specutils.SpectralRegion`
         Regions of the spectrum to use in the fitting. If None, then the
         whole spectrum will be used in the fitting.
-
     Additional keyword arguments are passed directly into the call to the
     ``fitter``.
 
@@ -289,9 +312,7 @@ def fit_lines(spectrum, model, fitter=fitting.LevMarLSQFitter(),
        * The models in the list of ``model`` are added
           together and passed as a compound model to the
           `~astropy.modeling.fitting.Fitter` class instance.
-
     """
-
     #
     # If we are to exclude certain regions, then remove them.
     #
@@ -343,6 +364,8 @@ def fit_lines(spectrum, model, fitter=fitting.LevMarLSQFitter(),
         fit_model = _fit_lines(spectrum, model_guess, fitter,
                                exclude_regions, weights, model_window,
                                ignore_units, **kwargs)
+        if model_guess.name is not None:
+            fit_model.name = model_guess.name
 
         fitted_models.append(fit_model)
 
@@ -353,53 +376,14 @@ def fit_lines(spectrum, model, fitter=fitting.LevMarLSQFitter(),
 
 
 def _fit_lines(spectrum, model, fitter=fitting.LevMarLSQFitter(),
-               exclude_regions=None, weights=None, window=None, ignore_units=False,
-               **kwargs):
+               exclude_regions=None, weights=None, window=None,
+               ignore_units=False, **kwargs):
     """
     Fit the input model (initial conditions) to the spectrum.  Output will be
     the same model with the parameters set based on the fitting.
 
     spectrum, model -> model
-
-    Parameters
-    ----------
-    spectrum : Spectrum1D
-        The spectrum object over which the equivalent width will be calculated.
-
-    model: `~astropy.modeling.Model`
-        The model or that contain the initial guess.
-
-    exclude_regions : list of `~specutils.SpectralRegion`
-        List of regions to exclude in the fitting.
-
-    weights : list  (NOT IMPLEMENTED YET)
-        List of weights to define importance of fitting regions.
-
-    window : `~specutils.SpectralRegion` or list of `~specutils.SpectralRegion`
-        Regions of the spectrum to use in the fitting. If None, then the
-        whole spectrum will be used in the fitting.
-
-    ignore_units : bool
-        If True, then ignore any units on the input model parameters.
-        (This would effectively be assuming the model and spectrum have the same units.)
-
-    Returns
-    -------
-    model : Compound model of `~astropy.modeling.Model`
-        A compound model of models with fitted parameters.
-
-    Notes
-    -----
-       * Could add functionality to set the bounds in ``model`` if they are not set.
-
-       * Additional keyword arguments are passed directly into
-         the call to the ``fitter``.
-
     """
-
-    if weights is not None:
-        raise NotImplementedError('Weights are not yet implemented.')
-
     #
     # If we are to exclude certain regions, then remove them.
     #
@@ -407,12 +391,31 @@ def _fit_lines(spectrum, model, fitter=fitting.LevMarLSQFitter(),
     if exclude_regions is not None:
         spectrum = excise_regions(spectrum, exclude_regions)
 
+    if isinstance(weights, str):
+        if weights == 'unc':
+            uncerts = spectrum.uncertainty
+
+            if uncerts is not None:
+                weights = uncerts.array ** -2
+            else:
+                logging.warning("Uncertainty values are not defined, but are "
+                                "trying to be used in model fitting.")
+        else:
+            raise ValueError("Unrecognized value `%s` in keyword argument.",
+                             weights)
+    elif weights is not None:
+        # Assume that the weights argument is list-like
+        weights = np.array(weights)
+
+    mask = spectrum.mask
+
     dispersion = spectrum.spectral_axis
     dispersion_unit = spectrum.spectral_axis.unit
 
     flux = spectrum.flux
     flux_unit = spectrum.flux.unit
 
+
     #
     # Determine the window if it is not None.  There
     # are several options here:
@@ -426,42 +429,63 @@ def _fit_lines(spectrum, model, fitter=fitting.LevMarLSQFitter(),
     #
 
     # In this case the window defines the area around the center of each model
+    window_indices = None
     if window is not None and isinstance(window, (float, int)):
         center = model.mean
-        indices = np.nonzero((spectrum.spectral_axis >= center-window) & (spectrum.spectral_axis < center+window))
+        window_indices = np.nonzero((spectrum.spectral_axis >= center-window) &
+                                    (spectrum.spectral_axis < center+window))
 
-        dispersion = dispersion[indices]
-        flux = flux[indices]
-
-    # In this case the window is the start and end points of where we should fit
+    # In this case the window is the start and end points of where we
+    # should fit
     elif window is not None and isinstance(window, tuple):
-        indices = np.nonzero((dispersion >= window[0]) & (dispersion < window[1]))
-
-        dispersion = dispersion[indices]
-        flux = flux[indices]
+        window_indices = np.nonzero((dispersion >= window[0]) &
+                             (dispersion < window[1]))
 
+    # in this case the window is spectral regions that determine where
+    # to fit.
     elif window is not None and isinstance(window, SpectralRegion):
-        try:
-            idx1, idx2 = window.bounds
-            if idx1 == idx2:
-                raise Exception("Bad selected region.")
-            extracted_regions = extract_region(spectrum, window)
-            dispersion, flux = _combined_region_data(extracted_regions)
-            dispersion = dispersion * dispersion_unit
-            flux = flux * flux_unit
-        except ValueError as e:
-            return
+        idx1, idx2 = window.bounds
+        if idx1 == idx2:
+            raise IndexError("Tried to fit a region containing no pixels.")
+
+        # HACK WARNING! This uses the extract machinery to create a set of
+        # indices by making an "index spectrum"
+        # note that any unit will do but Jy is at least flux-y
+        # TODO: really the spectral region machinery should have the power
+        # to create a mask, and we'd just use that...
+        idxarr = np.arange(spectrum.flux.size).reshape(spectrum.flux.shape)
+        index_spectrum = Spectrum1D(spectral_axis=spectrum.spectral_axis,
+                                    flux=u.Quantity(idxarr, u.Jy, dtype=int))
+
+        extracted_regions = extract_region(index_spectrum, window)
+        if isinstance(extracted_regions, list):
+            if len(extracted_regions) == 0:
+                raise ValueError('The whole spectrum is windowed out!')
+            window_indices = np.concatenate([s.flux.value.astype(int) for s in extracted_regions])
+        else:
+            if len(extracted_regions.flux) == 0:
+                raise ValueError('The whole spectrum is windowed out!')
+            window_indices = extracted_regions.flux.value.astype(int)
+
+    if window_indices is not None:
+        dispersion = dispersion[window_indices]
+        flux = flux[window_indices]
+        if mask is not None:
+            mask = mask[window_indices]
+        if weights is not None:
+            weights = weights[window_indices]
 
     if flux is None or len(flux) == 0:
         raise Exception("Spectrum flux is empty or None.")
 
     input_spectrum = spectrum
 
-    spectrum = Spectrum1D(flux=flux.value * flux_unit,
-                          spectral_axis=dispersion.value * dispersion_unit,
-                          wcs=input_spectrum.wcs,
-                          velocity_convention=input_spectrum.velocity_convention,
-                          rest_value=input_spectrum.rest_value)
+    spectrum = Spectrum1D(
+        flux=flux.value * flux_unit,
+        spectral_axis=dispersion.value * dispersion_unit,
+        wcs=input_spectrum.wcs,
+        velocity_convention=input_spectrum.velocity_convention,
+        rest_value=input_spectrum.rest_value)
 
     #
     # Compound models with units can not be fit.
@@ -470,14 +494,21 @@ def _fit_lines(spectrum, model, fitter=fitting.LevMarLSQFitter(),
     # units and then remove the units
     #
 
-    model_unitless, dispersion_unitless, flux_unitless = _strip_units_from_model(model, spectrum, convert=not ignore_units)
+    model_unitless, dispersion_unitless, flux_unitless = \
+        _strip_units_from_model(model, spectrum, convert=not ignore_units)
 
     #
     # Do the fitting of spectrum to the model.
     #
+    if mask is not None:
+        nmask = ~mask
+        dispersion_unitless = dispersion_unitless[nmask]
+        flux_unitless = flux_unitless[nmask]
+        if weights is not None:
+            weights = weights[nmask]
 
-    fit_model_unitless = fitter(model_unitless, dispersion_unitless, flux_unitless,
-                                **kwargs)
+    fit_model_unitless = fitter(model_unitless, dispersion_unitless,
+                                flux_unitless, weights=weights, **kwargs)
 
     #
     # Now add the units back onto the model....
@@ -486,52 +517,41 @@ def _fit_lines(spectrum, model, fitter=fitting.LevMarLSQFitter(),
     if not ignore_units:
         fit_model = _add_units_to_model(fit_model_unitless, model, spectrum)
     else:
-        fit_model = QuantityModel(fit_model_unitless, spectrum.spectral_axis.unit, spectrum.flux.unit)
+        fit_model = QuantityModel(fit_model_unitless,
+                                  spectrum.spectral_axis.unit,
+                                  spectrum.flux.unit)
 
     return fit_model
 
-def _combined_region_data(spec):
-    if isinstance(spec, list):
 
-        # Merge sub-spec spectral_axis and flux values.
-        x = np.array([sv for subspec in spec if subspec is not None
-                         for sv in subspec.spectral_axis.value])
-        y = np.array([sv for subspec in spec if subspec is not None
-                         for sv in subspec.flux.value])
-    else:
-        if spec is None:
-            return
-        x = spec.spectral_axis.value
-        y = spec.flux.value
+def _convert(quantity, dispersion_unit, dispersion, flux_unit):
+    """
+    Convert the quantity to the spectrum's units, and then we will use
+    the *value* of it in the new unitless-model.
+    """
+    with u.set_enabled_equivalencies(u.spectral()):
+        if quantity.unit.is_equivalent(dispersion_unit):
+            quantity = quantity.to(dispersion_unit)
 
-    if len(x) == 0:
-        return
-
-    return x, y
-
-def _convert(q, dispersion_unit, dispersion, flux_unit):
-    #
-    # Convert the quantity to the spectrum's units, and then we will use
-    # the *value* of it in the new unitless-model.
-    #
-
-    if q.unit.is_equivalent(dispersion_unit, equivalencies=u.equivalencies.spectral()):
-        quantity = q.to(dispersion_unit, equivalencies=u.equivalencies.spectral())
-
-    elif q.unit.is_equivalent(flux_unit, equivalencies=u.equivalencies.spectral_density(dispersion)):
-        quantity = q.to(flux_unit, equivalencies=u.equivalencies.spectral_density(dispersion))
+    with u.set_enabled_equivalencies(u.spectral_density(dispersion)):
+        if quantity.unit.is_equivalent(flux_unit):
+            quantity = quantity.to(flux_unit)
 
     return quantity
 
-def _convert_and_dequantify(poss_quantity, dispersion_unit, dispersion, flux_unit, convert=True):
+
+def _convert_and_dequantify(poss_quantity, dispersion_unit, dispersion,
+                            flux_unit, convert=True):
     """
     This method will convert the ``poss_quantity`` value to the proper
     dispersion or flux units and then strip the units.
 
-    If the ``poss_quantity`` is None, or a number, we just return that...
+    If the ``poss_quantity`` is None, or a number, we just return that.
 
-    Note: This method can be removed along with most of the others here
-          when astropy.fitting will fit models that contain units.
+    Notes
+    -----
+        This method can be removed along with most of the others here
+        when astropy.fitting will fit models that contain units.
 
     """
 
@@ -553,23 +573,26 @@ def _convert_and_dequantify(poss_quantity, dispersion_unit, dispersion, flux_uni
 
     return v
 
+
 def _strip_units_from_model(model_in, spectrum, convert=True):
     """
     This method strips the units from the model, so the result can
     be passed to the fitting routine. This is necessary as CoumpoundModel
     with units does not work in the fitters.
 
-    Note:  When CompoundModel with units works in the fitters this method
-           can be removed.
+    Notes
+    -----
+        When CompoundModel with units works in the fitters this method
+        can be removed.
 
-    Note: This assumes there are two types of models, those that are
-          based on `~astropy.modeling.models.PolynomialModel` and therefore
-          require the ``degree`` parameter when instantiating the class, and
-          "everything else" that does not require an "extra" parameter for
-          class instantiation.
+        This assumes there are two types of models, those that are
+        based on `~astropy.modeling.models.PolynomialModel` and therefore
+        require the ``degree`` parameter when instantiating the class, and
+        "everything else" that does not require an "extra" parameter for
+        class instantiation.
 
-    Note: If convert is False, then we will *not* do the conversion of units
-          to the units of the Spectrum1D object.  Otherwise we will convert.
+        If convert is False, then we will *not* do the conversion of units
+        to the units of the Spectrum1D object.  Otherwise we will convert.
     """
 
     #
@@ -585,8 +608,7 @@ def _strip_units_from_model(model_in, spectrum, convert=True):
     #
     # Determine if a compound model
     #
-
-    compound_model = model_in.n_submodels() > 1
+    compound_model = model_in.n_submodels > 1
 
     if not compound_model:
         # For this we are going to just make it a list so that we
@@ -596,8 +618,7 @@ def _strip_units_from_model(model_in, spectrum, convert=True):
         # If it is a compound model then we are going to create the RPN
         # representation of it which is a list that contains either astropy
         # models or string representations of operators (e.g., '+' or '*').
-        model_in = [c.value for c in model_in._tree.traverse_postorder()]
-
+        model_in = model_in.traverse_postorder(include_operator=True)
     #
     # Run through each model in the list or compound model
     #
@@ -618,14 +639,13 @@ def _strip_units_from_model(model_in, spectrum, convert=True):
         #
 
         if isinstance(sub_model, models.PolynomialModel):
-            new_sub_model = sub_model.__class__(sub_model.degree)
+            new_sub_model = sub_model.__class__(sub_model.degree, name=sub_model.name)
         else:
-            new_sub_model = sub_model.__class__()
+            new_sub_model = sub_model.__class__(name=sub_model.name)
 
-        #
         # Now for each parameter in the model determine if a dispersion or
-        # flux type of unit, then convert to spectrum units and then get the value.
-        #
+        # flux type of unit, then convert to spectrum units and then
+        # get the value.
 
         for pn in new_sub_model.param_names:
 
@@ -636,15 +656,14 @@ def _strip_units_from_model(model_in, spectrum, convert=True):
             # Add this information for the parameter name into the
             # new sub model.
             #
-
             setattr(new_sub_model, pn, v)
 
             #
             # Copy over all the constraints (e.g., tied, fixed...)
             #
-            for k, v in sub_model._constraints.items():
-                new_sub_model._constraints[k] = v
-
+            for constraint in ('tied', 'fixed'):
+                for k, v in getattr(sub_model, constraint).items():
+                    getattr(new_sub_model, constraint)[k] = v
             #
             # Convert teh bounds parameter
             #
@@ -655,9 +674,8 @@ def _strip_units_from_model(model_in, spectrum, convert=True):
 
             new_sub_model.bounds[pn] = tuple(new_bounds)
 
-
-        # The new model now has unitless information in it but has
-        # been converted to spectral unit scale.
+        # The new model now has unitless information in it but has been
+        # converted to spectral unit scale.
         model_out_stack.append(new_sub_model)
 
     # If a compound model we need to re-create it, otherwise
@@ -677,14 +695,16 @@ def _add_units_to_model(model_in, model_orig, spectrum):
     model passed in.  This is necessary as CoumpoundModel
     with units does not work in the fitters.
 
-    Note:  When CompoundModel with units works in the fitters this method
-           can be removed.
+    Notes
+    -----
+        When CompoundModel with units works in the fitters this method
+        can be removed.
 
-    Note: This assumes there are two types of models, those that are
-          based on `~astropy.modeling.models.PolynomialModel` and therefore
-          require the ``degree`` parameter when instantiating the class, and
-          "everything else" that does not require an "extra" parameter for
-          class instantiation.
+        This assumes there are two types of models, those that are
+        based on `~astropy.modeling.models.PolynomialModel` and therefore
+        require the ``degree`` parameter when instantiating the class, and
+        "everything else" that does not require an "extra" parameter for
+        class instantiation.
     """
 
     dispersion = spectrum.spectral_axis
@@ -694,15 +714,15 @@ def _add_units_to_model(model_in, model_orig, spectrum):
     # list so we can use the for loop below.
     #
 
-    compound_model = model_in.n_submodels() > 1
+    compound_model = model_in.n_submodels > 1
     if not compound_model:
         model_in_list = [model_in]
         model_orig_list = [model_orig]
     else:
         compound_model_in = model_in
 
-        model_in_list = [c.value for c in model_in._tree.traverse_postorder()]
-        model_orig_list = [c.value for c in model_orig._tree.traverse_postorder()]
+        model_in_list = model_in.traverse_postorder(include_operator=True)
+        model_orig_list = model_orig.traverse_postorder(include_operator=True)
 
     model_out_stack = []
     model_index = 0
@@ -736,9 +756,9 @@ def _add_units_to_model(model_in, model_orig, spectrum):
         #
 
         if isinstance(m_in, models.PolynomialModel):
-            new_sub_model = m_in.__class__(m_in.degree)
+            new_sub_model = m_in.__class__(m_in.degree, name=m_in.name)
         else:
-            new_sub_model = m_in.__class__()
+            new_sub_model = m_in.__class__(name=m_in.name)
 
         #
         # Convert the model values from the spectrum units back to the
@@ -764,8 +784,9 @@ def _add_units_to_model(model_in, model_orig, spectrum):
                 if m_orig_param_quantity.unit.is_equivalent(spectrum.spectral_axis.unit,
                                                             equivalencies=u.equivalencies.spectral()):
 
-                    # If it is a compound model, then we need to get the value from the
-                    # actual compound model as the tree is not updated in the fitting
+                    # If it is a compound model, then we need to get the value
+                    # from the actual compound model as the tree is not
+                    # updated in the fitting
                     if compound_model:
                         current_value = getattr(compound_model_in, '{}_{}'.format(pn, model_index)).value *\
                                         spectrum.spectral_axis.unit
@@ -779,8 +800,9 @@ def _add_units_to_model(model_in, model_orig, spectrum):
                 #
                 elif m_orig_param_quantity.unit.is_equivalent(spectrum.flux.unit,
                                                               equivalencies=u.equivalencies.spectral_density(dispersion)):
-                    # If it is a compound model, then we need to get the value from the
-                    # actual compound model as the tree is not updated in the fitting
+                    # If it is a compound model, then we need to get the value
+                    # from the actual compound model as the tree is not
+                    # updated in the fitting
                     if compound_model:
                         current_value = getattr(compound_model_in, '{}_{}'.format(pn, model_index)).value *\
                                         spectrum.flux.unit
@@ -789,6 +811,13 @@ def _add_units_to_model(model_in, model_orig, spectrum):
 
                     v = current_value.to(m_orig_param_quantity.unit,
                                          equivalencies=u.equivalencies.spectral_density(dispersion))
+                else:
+                    raise ValueError(
+                        "The parameter '{}' with unit '{}' is not convertible "
+                        "to either the current flux unit '{}' or spectral "
+                        "axis unit '{}'.".format(
+                            m_orig_param.name, m_orig_param.unit,
+                            spectrum.flux.unit, spectrum.spectral_axis.unit))
 
             else:
                 v = getattr(m_in, pn).value
@@ -802,8 +831,9 @@ def _add_units_to_model(model_in, model_orig, spectrum):
             #
             # Copy over all the constraints (e.g., tied, fixed, bounds...)
             #
-            for k, v in m_orig._constraints.items():
-                new_sub_model._constraints[k] = v
+            for constraint in ('tied', 'bounds', 'fixed'):
+                for k, v in getattr(m_orig, constraint).items():
+                    getattr(new_sub_model, constraint)[k] = v
 
         #
         # Add the new unit-filled model onto the stack.
@@ -824,23 +854,25 @@ def _add_units_to_model(model_in, model_orig, spectrum):
     else:
         model_out = model_out_stack[0]
 
-    # If the first parameter is not a Quantity, then at this point we will assume
-    # none of them are. (It would be inconsistent for fitting to have a model that
-    # has some parameters as Quantities and some values).
+    # If the first parameter is not a Quantity, then at this point we will
+    # assume none of them are. (It would be inconsistent for fitting to have
+    # a model that has some parameters as Quantities and some values).
     if getattr(model_orig, model_orig.param_names[0]).unit is None:
-        model_out = QuantityModel(model_out, spectrum.spectral_axis.unit, spectrum.flux.unit)
+        model_out = QuantityModel(model_out,
+                                  spectrum.spectral_axis.unit,
+                                  spectrum.flux.unit)
 
     return model_out
 
 
 def _combine_postfix(equation):
     """
-    Given a Python list in post order (RPN) of an equation, convert/apply the operations to evaluate.
-    The list order is the same as what is output from ``model._tree.traverse_postorder()``.
+    Given a Python list in post order (RPN) of an equation, convert/apply the
+    operations to evaluate. The list order is the same as what is output from
+    ``model._tree.traverse_postorder()``.
 
     Structure modified from https://codereview.stackexchange.com/questions/79795/reverse-polish-notation-calculator-in-python
     """
-
     ops = {'+': operator.add,
            '-': operator.sub,
            '*': operator.mul,
@@ -850,6 +882,7 @@ def _combine_postfix(equation):
 
     stack = []
     result = 0
+
     for i in equation:
         if isinstance(i, Model):
             stack.insert(0, i)
@@ -862,4 +895,5 @@ def _combine_postfix(equation):
                 n2 = stack.pop(0)
                 result = ops[i](n1, n2)
                 stack.insert(0, result)
+
     return result
