@@ -2,8 +2,7 @@
 The core specutils data objects package. This contains the
 `~astropy.nddata.NDData`-inherited classes used for storing the spectrum data.
 """
-from __future__ import absolute_import
-
 from .spectrum1d import *  # noqa
 from .spectral_region import *  # noqa
 from .spectrum_collection import *  #noqa
+from .spectrum_list import * #noqa
