@@ -1,12 +1,13 @@
 import logging
 from copy import deepcopy
 
-import numpy as np
-from astropy.wcs import WCSSUB_SPECTRAL
-from astropy.units import Unit
-from astropy import units as u
 import astropy.units.equivalencies as eq
+import numpy as np
+from astropy import units as u
 from astropy.utils.decorators import lazyproperty
+from astropy.wcs.wcsapi import HighLevelWCSWrapper
+
+from specutils.utils.wcs_utils import gwcs_from_array
 
 DOPPLER_CONVENTIONS = {}
 DOPPLER_CONVENTIONS['radio'] = u.doppler_radio
@@ -51,6 +52,9 @@ class OneDSpectrumMixin:
 
     @property
     def spectral_wcs(self):
+        """
+        Returns the spectral axes of the WCS
+        """
         return self.wcs.axes.spectral
 
     @lazyproperty
@@ -58,17 +62,19 @@ class OneDSpectrumMixin:
         """
         Returns a Quantity array with the values of the spectral axis.
         """
-
         if len(self.flux) > 0:
             spectral_axis = self.wcs.pixel_to_world(np.arange(self.flux.shape[-1]))
         else:
-            # After some discussion it was suggested to create the empty spectral
-            # axis this way to better use the WCS infrastructure. This is to prepare
-            # for a future where pixel_to_world might yield something more than
-            # just a raw Quantity, which is planned for the mid-term in astropy and
-            # possible gwcs.  Such changes might necessitate a revisit of this code.
-            dummy_spectrum = self.__class__(spectral_axis=[1,2]*self.wcs.spectral_axis_unit,
-                                            flux=[1,2]*self.flux.unit)
+            # After some discussion it was suggested to create the empty
+            # spectral axis this way to better use the WCS infrastructure.
+            # This is to prepare for a future where pixel_to_world might yield
+            # something more than just a raw Quantity, which is planned for
+            # the mid-term in astropy and possible gwcs.  Such changes might
+            # necessitate a revisit of this code.
+            dummy_spectrum = self.__class__(
+                spectral_axis=[1, 2] * self.spectral_axis_unit,
+                flux=[1, 2] * self.flux.unit)
+
             spectral_axis = dummy_spectrum.wcs.pixel_to_world([0])[1:]
 
         return spectral_axis
@@ -78,7 +84,10 @@ class OneDSpectrumMixin:
         """
         Returns the units of the spectral axis.
         """
-        return self.wcs.spectral_axis_unit
+        if isinstance(self.wcs, HighLevelWCSWrapper):
+            return u.Unit(self.wcs.world_axis_units[0])
+
+        return self.wcs.unit[0]
 
     @property
     def flux(self):
@@ -100,7 +109,7 @@ class OneDSpectrumMixin:
         Parameters
         ----------
         unit : str or `~astropy.units.Unit`
-            The unit to conver the flux array to.
+            The unit to convert the flux array to.
 
         equivalencies : list of equivalencies
             Custom equivalencies to apply to conversions.
@@ -116,6 +125,7 @@ class OneDSpectrumMixin:
             A new spectrum with the converted flux array
         """
         new_spec = deepcopy(self)
+
         if not suppress_conversion:
             if equivalencies is None:
                 equivalencies = eq.spectral_density(self.spectral_axis)
@@ -132,6 +142,9 @@ class OneDSpectrumMixin:
 
     @property
     def velocity_convention(self):
+        """
+        Returns the velocity convention
+        """
         return self._velocity_convention
 
     def with_velocity_convention(self, velocity_convention):
@@ -167,6 +180,9 @@ class OneDSpectrumMixin:
             (wavelength, energy, frequency, wave number).
         type : {"doppler_relativistic", "doppler_optical", "doppler_radio"}
             The type of doppler spectral equivalency.
+        redshift or radial_velocity
+            If present, this shift is applied to the final output velocity to
+            get into the rest frame of the object.
 
         Returns
         -------
@@ -185,6 +201,10 @@ class OneDSpectrumMixin:
 
         new_data = self.spectral_axis.to(u.km/u.s, equivalencies=equiv)
 
+        # if redshift/rv is present, apply it:
+        if self._radial_velocity is not None:
+            new_data += self.radial_velocity
+
         return new_data
 
     def with_spectral_unit(self, unit, velocity_convention=None,
@@ -280,14 +300,15 @@ class OneDSpectrumMixin:
         meta = self._meta.copy()
 
         if 'original_unit' not in self._meta:
-            meta['original_unit'] = self.wcs.spectral_axis_unit
+            meta['original_unit'] = self.wcs.unit[0]
 
         # Create the new wcs object
-        new_wcs = self.wcs.with_spectral_unit(unit=unit,
-                                         rest_value=rest_value,
-                                         velocity_convention=velocity_convention)
+        if isinstance(unit, u.UnitBase) and unit.is_equivalent(
+                self.wcs.unit[0], equivalencies=u.spectral()):
+            return gwcs_from_array(self.spectral_axis), meta
 
-        return new_wcs, meta
+        logging.error("WCS units incompatible: {} and {}.".format(
+            unit, self._wcs_unit))
 
 
 class InplaceModificationMixin:
