@@ -6,10 +6,13 @@ from astropy.modeling import models
 from astropy.nddata import StdDevUncertainty
 from astropy.stats.funcs import gaussian_sigma_to_fwhm
 from astropy.tests.helper import quantity_allclose
+from astropy.utils.exceptions import AstropyUserWarning
 
 from ..spectra import Spectrum1D, SpectralRegion
 from ..analysis import (line_flux, equivalent_width, snr, centroid,
-                        gaussian_sigma_width, gaussian_fwhm, fwhm)
+                        gaussian_sigma_width, gaussian_fwhm, fwhm,
+                        snr_derived, fwzi, is_continuum_below_threshold)
+from ..fitting import find_lines_threshold
 from ..tests.spectral_examples import simulated_spectra
 
 
@@ -48,7 +51,7 @@ def test_equivalent_width():
 
     result = equivalent_width(spectrum)
 
-    assert result.unit.is_equivalent(spectrum.spectral_axis_unit)
+    assert result.unit.is_equivalent(spectrum.wcs.unit)
 
     # Since this is an emission line, we expect the equivalent width value to
     # be negative
@@ -89,7 +92,7 @@ def test_equivalent_width_continuum(continuum):
 
     result = equivalent_width(spectrum, continuum=continuum)
 
-    assert result.unit.is_equivalent(spectrum.spectral_axis_unit)
+    assert result.unit.is_equivalent(spectrum.wcs.unit)
 
     # Since this is an emission line, we expect the equivalent width value to
     # be negative
@@ -113,7 +116,7 @@ def test_equivalent_width_absorption():
 
     result = equivalent_width(spectrum)
 
-    assert result.unit.is_equivalent(spectrum.spectral_axis_unit)
+    assert result.unit.is_equivalent(spectrum.wcs.unit)
 
     # Since this is an absorption line, we expect the equivalent width value to
     # be positive
@@ -267,6 +270,22 @@ def test_snr_two_regions(simulated_spectra):
     assert np.allclose(spec_snr, spec_snr_expected)
 
 
+def test_snr_derived():
+    np.random.seed(42)
+
+    x = np.arange(1, 101) * u.um
+    y = np.random.random(len(x))*u.Jy
+    spectrum = Spectrum1D(spectral_axis=x, flux=y)
+
+    assert np.allclose(snr_derived(spectrum), 1.604666860424951)
+
+    sr = SpectralRegion(38*u.um, 48*u.um)
+    assert np.allclose(snr_derived(spectrum, sr), 2.330463630828406)
+
+    sr2 = SpectralRegion(48*u.um, 57*u.um)
+    assert np.allclose(snr_derived(spectrum, [sr, sr2]), [2.330463630828406, 2.9673559890209305])
+
+
 def test_centroid(simulated_spectra):
     """
     Test the simple version of the spectral centroid.
@@ -459,14 +478,27 @@ def test_fwhm():
     expected = stddev * gaussian_sigma_to_fwhm
     assert quantity_allclose(result, expected, atol=0.01*u.GHz)
 
-
     # Highest point at the first point
     wavelengths = np.linspace(1, 10, 100) * u.um
-    flux = (1.0 / wavelengths.value)*u.Jy # highest point first.
+    flux = (1.0 / wavelengths.value) * u.Jy  # highest point first.
 
     spectrum = Spectrum1D(spectral_axis=wavelengths, flux=flux)
     result = fwhm(spectrum)
-    assert result == 0.9090909090909092*u.um
+    # Note that this makes a little more sense than the previous version;
+    # since the maximum value occurs at wavelength=1, and the half-value of
+    # flux (0.5) occurs at exactly wavelength=2, the result should be
+    # exactly 1 (2 - 1).
+    assert result == 1.0 * u.um
+
+    # Test the interpolation used in FWHM for wavelength values that are not
+    # on the grid
+    wavelengths = np.linspace(1, 10, 31) * u.um
+    flux = (1.0 / wavelengths.value) * u.Jy  # highest point first.
+
+    spectrum = Spectrum1D(spectral_axis=wavelengths, flux=flux)
+    result = fwhm(spectrum)
+
+    assert quantity_allclose(result, 1.01 * u.um)
 
     # Highest point at the last point
     wavelengths = np.linspace(1, 10, 100) * u.um
@@ -507,3 +539,91 @@ def test_fwhm_multi_spectrum():
 
     expected = stddevs * gaussian_sigma_to_fwhm
     assert quantity_allclose(results, expected, atol=0.01*u.GHz)
+
+
+def test_fwzi():
+    np.random.seed(42)
+
+    disp = np.linspace(0, 100, 1000) * u.AA
+
+    g = models.Gaussian1D(mean=np.mean(disp),
+                          amplitude=1 * u.Jy,
+                          stddev=2 * u.AA)
+
+    flux = g(disp)
+    flux_with_noise = g(disp) + ((np.random.sample(disp.size) - 0.5) * 0.1) * u.Jy
+
+    spec = Spectrum1D(spectral_axis=disp, flux=flux)
+    spec_with_noise = Spectrum1D(spectral_axis=disp, flux=flux_with_noise)
+
+    assert quantity_allclose(fwzi(spec), 226.89732509 * u.AA)
+    assert quantity_allclose(fwzi(spec_with_noise), 106.99998944 * u.AA)
+
+
+def test_fwzi_multi_spectrum():
+    np.random.seed(42)
+
+    disp = np.linspace(0, 100, 1000) * u.AA
+
+    amplitudes = [0.1, 1, 10] * u.Jy
+    means = [25, 50, 75] * u.AA
+    stddevs = [1, 5, 10] * u.AA
+    params = list(zip(amplitudes, means, stddevs))
+
+    flux = np.zeros(shape=(3, len(disp)))
+
+    for i in range(3):
+        flux[i] = models.Gaussian1D(*params[i])(disp)
+
+    spec = Spectrum1D(spectral_axis=disp, flux=flux * u.Jy)
+
+    expected = [113.51706001 * u.AA, 567.21252727 * u.AA, 499.5024546 * u.AA]
+
+    assert quantity_allclose(fwzi(spec), expected)
+
+
+def test_is_continuum_below_threshold():
+
+    # No mask, no uncertainty
+    wavelengths = [300, 500, 1000] * u.nm
+    data = [0.001, -0.003, 0.003] * u.Jy
+    spectrum = Spectrum1D(spectral_axis=wavelengths, flux=data)
+    assert True == is_continuum_below_threshold(spectrum, threshold=0.1*u.Jy)
+
+#    # No mask, no uncertainty, threshold is float
+#    wavelengths = [300, 500, 1000] * u.nm
+#    data = [0.0081, 0.0043, 0.0072] * u.Jy
+#    spectrum = Spectrum1D(spectral_axis=wavelengths, flux=data)
+#    assert True == is_continuum_below_threshold(spectrum, threshold=0.1)
+
+    # No mask, with uncertainty
+    wavelengths = [300, 500, 1000] * u.nm
+    data = [0.03, 0.029, 0.031] * u.Jy
+    uncertainty = StdDevUncertainty([1.01, 1.03, 1.01] * u.Jy)
+    spectrum = Spectrum1D(spectral_axis=wavelengths, flux=data,
+                          uncertainty=uncertainty)
+
+    assert True == is_continuum_below_threshold(spectrum, threshold=0.1*u.Jy)
+
+    # With mask, with uncertainty
+    wavelengths = [300, 500, 1000] * u.nm
+    data = [0.01, 1.029, 0.013] * u.Jy
+    mask = np.array([False, True, False])
+    uncertainty = StdDevUncertainty([1.01, 1.13, 1.1] * u.Jy)
+    spectrum = Spectrum1D(spectral_axis=wavelengths, flux=data,
+                          uncertainty=uncertainty, mask=mask)
+
+    assert True == is_continuum_below_threshold(spectrum, threshold=0.1*u.Jy)
+
+    # With mask, with uncertainty -- should throw an exception
+    wavelengths = [300, 500, 1000] * u.nm
+    data = [10.03, 10.029, 10.033] * u.Jy
+    mask = np.array([False, False, False])
+    uncertainty = StdDevUncertainty([1.01, 1.13, 1.1] * u.Jy)
+    spectrum = Spectrum1D(spectral_axis=wavelengths, flux=data,
+                          uncertainty=uncertainty, mask=mask)
+
+    print('spectrum has flux {}'.format(spectrum.flux))
+    with pytest.warns(AstropyUserWarning) as e_info:
+        find_lines_threshold(spectrum, noise_factor=1)
+        assert len(e_info)==1 and 'if you want to suppress this warning' in e_info[0].message.args[0].lower()
