@@ -1,13 +1,14 @@
-import numpy as np
-
 import astropy.units as u
+import numpy as np
 from astropy.modeling import models
+from astropy.nddata import StdDevUncertainty
+from astropy.tests.helper import assert_quantity_allclose
 
-from ..spectra import Spectrum1D, SpectralRegion
+from ..analysis import fwhm, centroid
 from ..fitting import (fit_lines, find_lines_derivative,
                        find_lines_threshold, estimate_line_parameters)
-from ..analysis import fwhm, centroid
-from ..manipulation import noise_region_uncertainty
+from ..manipulation import noise_region_uncertainty, spectrum_from_model
+from ..spectra import Spectrum1D, SpectralRegion
 
 
 def single_peak():
@@ -100,13 +101,16 @@ def test_single_peak_estimate():
 
     #
     # Estimate parameter Gaussian1D
+    # we give the true values for the Gaussian because it actually *should*
+    # be pretty close to the true values, because it's a Gaussian...
     #
 
     g_init = estimate_line_parameters(s_single, models.Gaussian1D())
 
-    assert np.isclose(g_init.amplitude.value, 3.354169257846847)
-    assert np.isclose(g_init.mean.value, 6.218588636687762)
-    assert np.isclose(g_init.stddev.value, 1.608040201005025)
+
+    assert np.isclose(g_init.amplitude.value, 3., rtol=.2)
+    assert np.isclose(g_init.mean.value, 6.3, rtol=.1)
+    assert np.isclose(g_init.stddev.value, 0.8, rtol=.3)
 
     assert g_init.amplitude.unit == u.Jy
     assert g_init.mean.unit == u.um
@@ -114,13 +118,16 @@ def test_single_peak_estimate():
 
     #
     # Estimate parameter Lorentz1D
+    # unlike the Gaussian1D here we do hand-picked comparison values, because
+    # the "single peak" is a Gaussian and therefore the Lorentzian fit shouldn't
+    # be quite right anyway
     #
 
     g_init = estimate_line_parameters(s_single, models.Lorentz1D())
 
     assert np.isclose(g_init.amplitude.value, 3.354169257846847)
     assert np.isclose(g_init.x_0.value, 6.218588636687762)
-    assert np.isclose(g_init.fwhm.value, 1.608040201005025)
+    assert np.isclose(g_init.fwhm.value, 1.6339001193853715)
 
     assert g_init.amplitude.unit == u.Jy
     assert g_init.x_0.unit == u.um
@@ -134,8 +141,8 @@ def test_single_peak_estimate():
 
     assert np.isclose(g_init.amplitude_L.value, 3.354169257846847)
     assert np.isclose(g_init.x_0.value, 6.218588636687762)
-    assert np.isclose(g_init.fwhm_L.value, 1.1370561305512321)
-    assert np.isclose(g_init.fwhm_G.value, 1.1370561305512321)
+    assert np.isclose(g_init.fwhm_L.value, 1.1553418541989058)
+    assert np.isclose(g_init.fwhm_G.value, 1.1553418541989058)
 
     assert g_init.amplitude_L.unit == u.Jy
     assert g_init.x_0.unit == u.um
@@ -144,25 +151,28 @@ def test_single_peak_estimate():
 
 
     #
-    # Estimate parameter MexicanHat1D
+    # Estimate parameter RickerWavelet1D
     #
-    mh = models.MexicanHat1D()
+    mh = models.RickerWavelet1D
     estimators = {
         'amplitude': lambda s: max(s.flux),
         'x_0': lambda s: centroid(s, region=None),
-        'stddev': lambda s: fwhm(s)
+        'sigma': lambda s: fwhm(s)
     }
-    mh._constraints['parameter_estimator'] = estimators
+    #mh._constraints['parameter_estimator'] = estimators
+    mh.amplitude.estimator = lambda s: max(s.flux)
+    mh.x_0.estimator = lambda s: centroid(s, region=None)
+    mh.sigma.estimator = lambda s: fwhm(s)
 
     g_init = estimate_line_parameters(s_single, mh)
 
     assert np.isclose(g_init.amplitude.value, 3.354169257846847)
     assert np.isclose(g_init.x_0.value, 6.218588636687762)
-    assert np.isclose(g_init.stddev.value, 1.608040201005025)
+    assert np.isclose(g_init.sigma.value, 1.6339001193853715)
 
     assert g_init.amplitude.unit == u.Jy
     assert g_init.x_0.unit == u.um
-    assert g_init.stddev.unit == u.um
+    assert g_init.sigma.unit == u.um
 
 
 def test_single_peak_fit():
@@ -189,6 +199,46 @@ def test_single_peak_fit():
     assert np.allclose(y_single_fit.value[::10], y_single_fit_expected.value, atol=1e-5)
 
 
+def test_single_peak_fit_with_uncertainties():
+    """
+    Single peak fit
+    """
+
+    # Create the spectrum
+    line_mod = models.Gaussian1D(amplitude=100,
+                                 mean=6563*u.angstrom,
+                                 stddev=20*u.angstrom) + models.Const1D(10)
+
+    init_mod = models.Gaussian1D(amplitude=85 * u.Jy,
+                                 mean=6550*u.angstrom,
+                                 stddev=30*u.angstrom) + models.Const1D(8 * u.Jy)
+    x = np.linspace(6400, 6700, 300) * u.AA
+
+    def calculate_rms(x, init_mod, implicit_weights):
+        rms = []
+
+        for _ in range(100):
+            ymod = line_mod(x)
+            y = np.random.poisson(ymod)
+            unc = np.sqrt(ymod)
+
+            spec = Spectrum1D(spectral_axis=x, flux=y * u.Jy,
+                              uncertainty=StdDevUncertainty(unc * u.Jy))
+
+            weights = 'unc' if implicit_weights else unc ** -2
+
+            spec_fit = fit_lines(spec, init_mod, weights=weights)
+
+            rms.append(np.std(spec_fit(x).value - y))
+
+        return np.median(rms)
+
+    assert np.allclose(calculate_rms(x, init_mod, implicit_weights=True),
+                       5.113708262419985)
+    assert np.allclose(calculate_rms(x, init_mod, implicit_weights=False),
+                       5.147348340711497)
+
+
 def test_single_peak_fit_window():
     """
     Single Peak fit with a window specified
@@ -420,19 +470,41 @@ def test_fixed_parameters():
     # Test passing fixed and bounds parameters
     g_init = models.Gaussian1D(amplitude=3.*u.Jy, mean=6.1*u.um, stddev=1.*u.um,
                                fixed={'mean': True},
-                               bounds={'amplitude': (2, 5)*u.Jy})
+                               bounds={'amplitude': (2, 5)*u.Jy},
+                               name="Gaussian Test Model")
 
     g_fit = fit_lines(spectrum, g_init)
 
-    assert g_fit.mean == 6.1*u.um
+    assert_quantity_allclose(g_fit.mean, 6.1*u.um)
     assert g_fit.bounds == g_init.bounds
+    assert g_fit.name == "Gaussian Test Model"
 
     # Test passing of tied parameter
-    g_init = models.Gaussian1D(amplitude=3.*u.Jy, mean=6.1*u.um, stddev=1.*u.um,
-                               tied={'mean': tie_center})
+    g_init = models.Gaussian1D(amplitude=3.*u.Jy, mean=6.1*u.um, stddev=1.*u.um)
+    g_init.mean.tied = tie_center
     g_fit = fit_lines(spectrum, g_init)
 
     assert g_fit.tied == g_init.tied
+    assert g_fit.name == g_init.name
+
+
+def test_name_preservation_after_fitting():
+    """
+    Test to confirm model and submodels names are preserved after fitting.
+    """
+
+    x = np.linspace(0., 10., 200)
+    y = 3 * np.exp(-0.5 * (x - 6.3)**2 / 0.8**2)
+    y += np.random.normal(0., 0.2, x.shape)
+    spectrum = Spectrum1D(flux=y*u.Jy, spectral_axis=x*u.um)
+
+    subcomponents = models.Gaussian1D(name="Model I") * models.Gaussian1D(name="Second Model")
+    c_model = subcomponents + models.Gaussian1D(name="Model 3")
+    c_model.name = "Compound Model with 3 components"
+    model_fit = fit_lines(spectrum, c_model)
+
+    assert model_fit.name == "Compound Model with 3 components"
+    assert model_fit.submodel_names == ("Model I", "Second Model", "Model 3")
 
 
 def test_ignore_units():
@@ -512,3 +584,97 @@ def test_fitter_parameters():
                2.88900005e-01, 6.24602556e-02, 9.22061121e-03, 9.29427266e-04]) * u.Jy
 
     assert np.allclose(y_single_fit.value[::10], y_single_fit_expected.value, atol=1e-5)
+
+
+def test_spectrum_from_model():
+    """
+    This test fits the the first simulated spectrum from the fixture.  The
+    initial guesses are manually set here with bounds that essentially make
+    sense as the functionality of the test is to make sure the fit works and
+    we get a reasonable answer out **given** good initial guesses.
+    """
+
+    np.random.seed(0)
+    x = np.linspace(0., 10., 200)
+    y = 3 * np.exp(-0.5 * (x - 6.3)**2 / 0.1**2)
+    y += np.random.normal(0., 0.2, x.shape)
+
+    y_continuum = 3.2 * np.exp(-0.5 * (x - 5.6)**2 / 4.8**2)
+    y += y_continuum
+
+    spectrum = Spectrum1D(flux=y*u.Jy, spectral_axis=x*u.um)
+
+    # Unitless test
+    chebyshev = models.Chebyshev1D(3, c0=0.1, c1=4, c2=5)
+    spectrum_chebyshev = spectrum_from_model(chebyshev, spectrum)
+
+    flux_expected = np.array([-4.90000000e+00, -3.64760991e-01,  9.22085553e+00,  2.38568496e+01,
+                              4.35432211e+01,  6.82799702e+01,  9.80670968e+01,  1.32904601e+02,
+                              1.72792483e+02,  2.17730742e+02,  2.67719378e+02,  3.22758392e+02,
+                              3.82847784e+02,  4.47987553e+02,  5.18177700e+02,  5.93418224e+02,
+                              6.73709126e+02,  7.59050405e+02,  8.49442062e+02,  9.44884096e+02])
+
+    assert np.allclose(spectrum_chebyshev.flux.value[::10], flux_expected, atol=1e-5)
+
+    # Unitfull test
+    gaussian = models.Gaussian1D(amplitude=5*u.Jy, mean=4*u.um, stddev=2.3*u.um)
+    spectrum_gaussian = spectrum_from_model(gaussian, spectrum)
+
+    flux_expected = np.array([1.1020263, 1.57342489, 2.14175093, 2.77946243, 3.4389158,
+                              4.05649712, 4.56194132, 4.89121902, 4.99980906, 4.872576,
+                              4.52723165, 4.01028933, 3.3867847, 2.72689468, 2.09323522,
+                              1.5319218, 1.06886794, 0.71101768, 0.45092638, 0.27264641])
+
+    assert np.allclose(spectrum_gaussian.flux.value[::10], flux_expected, atol=1e-5)
+
+
+def test_masking():
+    """
+    Test fitting spectra with masks
+    """
+    wl, flux = double_peak()
+    s = Spectrum1D(flux=flux*u.Jy, spectral_axis=wl*u.um)
+
+    # first we fit a single gaussian to the double_peak model, using the
+    # known-good second peak (but a bit higher in amplitude). It should lock
+    # in on the *second* peak since it's already close:
+    g_init = models.Gaussian1D(2.5, 5.5, 0.2)
+    g_fit1 = fit_lines(s, g_init)
+    assert u.allclose(g_fit1.mean, 5.5, atol=.1)
+
+    # now create a spectrum where the region around the second peak is masked.
+    # The fit should now go to the *first* peak
+    s_msk = Spectrum1D(flux=flux*u.Jy, spectral_axis=wl*u.um, mask=(5.1 < wl)&(wl < 6.1))
+    g_fit2 = fit_lines(s_msk, g_init)
+    assert u.allclose(g_fit2.mean, 4.6, atol=.1)
+
+    # double check that it works with weights as well
+    g_fit3 = fit_lines(s_msk, g_init, weights=np.ones_like(s_msk.flux.value))
+    assert g_fit2.mean == g_fit3.mean
+
+
+def test_window_extras():
+    """
+    Test that fitting works with masks and weights when a window is present
+    """
+    # similar to the masking test, but add a broad window around the whole thing
+    wl, flux = double_peak()
+    g_init = models.Gaussian1D(2.5, 5.5, 0.2)
+    window_region = SpectralRegion(4*u.um, 8*u.um)
+    mask = (5.1 < wl) & (wl < 6.1)
+
+    s_msk = Spectrum1D(flux=flux*u.Jy, spectral_axis=wl*u.um, mask=mask)
+
+    g_fit1 = fit_lines(s_msk, g_init, window=window_region)
+    assert u.allclose(g_fit1.mean, 4.6, atol=.1)
+
+    # check that if we weight instead of masking, we get the same result
+    s = Spectrum1D(flux=flux*u.Jy, spectral_axis=wl*u.um)
+    weights = (~mask).astype(float)
+    g_fit2 = fit_lines(s, g_init, weights=weights, window=window_region)
+    assert u.allclose(g_fit2.mean, 4.6, atol=.1)
+
+    # and the same with both together
+    weights = (~mask).astype(float)
+    g_fit3 = fit_lines(s_msk, g_init, weights=weights, window=window_region)
+    assert u.allclose(g_fit3.mean, 4.6, atol=.1)
