@@ -30,6 +30,11 @@ def test_create_from_arrays():
     assert isinstance(spec.flux, u.Quantity)
     assert spec.flux.size == 50
 
+    # Test creating spectrum with unknown arguments
+    with pytest.raises(ValueError) as e_info:
+        spec = Spectrum1D(wavelength=np.arange(1, 50) * u.nm,
+                          flux=np.random.randn(48) * u.Jy)
+
 
 def test_create_from_multidimensional_arrays():
     """
@@ -45,6 +50,11 @@ def test_create_from_multidimensional_arrays():
     assert (spec.frequency == freqs).all()
     assert (spec.flux == flux).all()
 
+    # Mis-matched lengths should raise an exception
+    freqs = np.arange(50) * u.GHz
+    flux = np.random.random((5, len(freqs)-1)) * u.Jy
+    with pytest.raises(ValueError) as e_info:
+        spec = Spectrum1D(spectral_axis=freqs, flux=flux)
 
 def test_create_from_quantities():
     spec = Spectrum1D(spectral_axis=np.arange(1, 50) * u.nm,
@@ -54,12 +64,17 @@ def test_create_from_quantities():
     assert spec.spectral_axis.unit == u.nm
     assert spec.spectral_axis.size == 49
 
+    # Mis-matched lengths should raise an exception
+    with pytest.raises(ValueError) as e_info:
+        spec = Spectrum1D(spectral_axis=np.arange(1, 50) * u.nm,
+                      flux=np.random.randn(48) * u.Jy)
+
 
 def test_create_implicit_wcs():
     spec = Spectrum1D(spectral_axis=np.arange(50) * u.AA,
                       flux=np.random.randn(50) * u.Jy)
 
-    assert isinstance(spec.wcs.wcs, gwcs.wcs.WCS)
+    assert isinstance(spec.wcs, gwcs.wcs.WCS)
 
     pix2world = spec.wcs.pixel_to_world(np.arange(5, 10))
 
@@ -71,7 +86,7 @@ def test_create_implicit_wcs_with_spectral_unit():
     spec = Spectrum1D(spectral_axis=np.arange(1, 50) * u.nm,
                       flux=np.random.randn(49) * u.Jy)
 
-    assert isinstance(spec.wcs.wcs, gwcs.wcs.WCS)
+    assert isinstance(spec.wcs, gwcs.wcs.WCS)
 
     pix2world = spec.wcs.pixel_to_world(np.arange(5, 10))
 
@@ -106,6 +121,63 @@ def test_spectral_axis_conversions():
     new_spec = spec.with_spectral_unit(u.GHz)
 
 
+def test_redshift():
+    spec = Spectrum1D(flux=np.array([26.0, 30., 44.5]) * u.Jy,
+                      spectral_axis=np.array([4000, 6000, 8000]) * u.AA,
+                      velocity_convention='optical',
+                      rest_value=6000 * u.AA)
+
+    assert u.allclose(spec.velocity, [-99930.8, 0, 99930.8]*u.km/u.s,
+                     atol=0.5*u.km/u.s)
+
+    spec = Spectrum1D(flux=np.array([26.0, 30., 44.5]) * u.Jy,
+                      spectral_axis=np.array([4000, 6000, 8000]) * u.AA,
+                      velocity_convention='optical',
+                      rest_value=6000 * u.AA,
+                      redshift= 0.1)
+
+    assert u.allclose(spec.velocity, [-69951.3, 29979.2, 129910.1]*u.km/u.s,
+                     atol=0.5*u.km/u.s)
+
+    #-------------------------
+
+    spec = Spectrum1D(flux=np.array([26.0, 30.0, 44.5]) * u.Jy,
+                      spectral_axis=np.array([10.5, 11.0, 11.5]) * u.GHz,
+                      velocity_convention='radio',
+                      rest_value=11.0 * u.GHz)
+
+    assert u.allclose(spec.velocity, [13626., 0, -13626]*u.km/u.s,
+                     atol=1*u.km/u.s)
+
+    spec = Spectrum1D(flux=np.array([26.0, 30.0, 44.5]) * u.Jy,
+                      spectral_axis=np.array([10.5, 11.0, 11.5]) * u.GHz,
+                      velocity_convention='radio',
+                      rest_value=11.0 * u.GHz,
+                      redshift= 0.1)
+
+    assert u.allclose(spec.velocity, [43606., 29979., 16352.]*u.km/u.s,
+                      atol=1*u.km/u.s)
+
+    #------------------------- radial velocity mode
+
+    spec = Spectrum1D(flux=np.array([26.0, 30., 44.5]) * u.Jy,
+                      spectral_axis=np.array([4000, 6000, 8000]) * u.AA,
+                      velocity_convention='optical',
+                      rest_value=6000 * u.AA)
+
+    assert u.allclose(spec.velocity, [-99930.8, 0.0, 99930.8]*u.km/u.s,
+                      atol=0.5*u.km/u.s)
+
+    spec = Spectrum1D(flux=np.array([26.0, 30., 44.5]) * u.Jy,
+                      spectral_axis=np.array([4000, 6000, 8000]) * u.AA,
+                      velocity_convention='optical',
+                      rest_value=6000 * u.AA,
+                      radial_velocity=1000.*u.km/u.s)
+
+    assert u.allclose(spec.velocity, [-98930.8, 1000.0, 100930.8]*u.km/u.s,
+                      atol=0.5*u.km/u.s)
+
+
 def test_flux_unit_conversion():
     # By default the flux units should be set to Jy
     s = Spectrum1D(flux=np.array([26.0, 44.5]) * u.Jy,
@@ -115,8 +187,8 @@ def test_flux_unit_conversion():
 
     # Simple Unit Conversion
     s = Spectrum1D(flux=np.array([26.0, 44.5]) * u.Jy, spectral_axis=np.array([400, 500])*u.nm)
-    converted_spec = s.new_flux_unit(unit=u.uJy)[0]
-    assert ((26.0 * u.Jy).to(u.uJy) == converted_spec.flux)
+    converted_spec = s.new_flux_unit(unit=u.uJy)
+    assert ((26.0 * u.Jy).to(u.uJy) == converted_spec.flux[0])
 
     # Make sure incompatible units raise UnitConversionError
     with pytest.raises(u.UnitConversionError):
@@ -128,8 +200,8 @@ def test_flux_unit_conversion():
     eq = [[u.Jy, u.m,
           lambda x: np.full_like(np.array(x), 1000.0, dtype=np.double),
           lambda x: np.full_like(np.array(x), 0.001, dtype=np.double)]]
-    converted_spec = s.new_flux_unit(unit=u.m, equivalencies=eq)[0]
-    assert 1000.0 * u.m == converted_spec.flux
+    converted_spec = s.new_flux_unit(unit=u.m, equivalencies=eq)
+    assert 1000.0 * u.m == converted_spec.flux[0]
 
     # Check if suppressing the unit conversion works
     s = Spectrum1D(flux=np.array([26.0, 44.5]) * u.Jy, spectral_axis=np.array([400, 500]) * u.nm)
@@ -142,28 +214,30 @@ def test_wcs_transformations():
     spec = Spectrum1D(spectral_axis=np.arange(1, 50) * u.nm,
                       flux=np.random.randn(49) * u.Jy)
 
-    pix_axis = spec.wcs.world_to_pixel(np.arange(20, 30))
-    disp_axis = spec.wcs.pixel_to_world(np.arange(20, 30) * u.nm)
+    pix_axis = spec.wcs.world_to_pixel(np.arange(20, 30) * u.nm)
+    disp_axis = spec.wcs.pixel_to_world(np.arange(20, 30))
 
     assert isinstance(pix_axis, np.ndarray)
     assert isinstance(disp_axis, u.Quantity)
 
     # Test transform with different unit
-    spec.wcs.world_to_pixel(np.arange(20, 30) * u.GHz)
+    with u.set_enabled_equivalencies(u.spectral()):
+        spec.wcs.world_to_pixel(np.arange(20, 30) * u.GHz)
 
     # Test with a FITS WCS
     my_wcs = fitswcs.WCS(header={'CDELT1': 1, 'CRVAL1': 6562.8, 'CUNIT1': 'Angstrom',
-                                 'CTYPE1': 'WAVE', 'RESTFRQ': 1400000000, 'CRPIX1': 25})
+                                 'CTYPE1': 'WAVE', 'RESTFRQ': 1400000000, 'CRPIX1': 25},
+                         naxis=1)
 
     spec = Spectrum1D(flux=[5,6,7] * u.Jy, wcs=my_wcs)
 
-    pix_axis = spec.wcs.world_to_pixel(np.arange(20, 30))
-    disp_axis = spec.wcs.pixel_to_world(np.arange(20, 30) * u.nm)
+    pix_axis = spec.wcs.world_to_pixel(20 * u.um)
+    disp_axis = spec.wcs.pixel_to_world(np.arange(20, 30))
 
     assert isinstance(pix_axis, np.ndarray)
     assert isinstance(disp_axis, u.Quantity)
 
-    assert np.allclose(spec.wcs.world_to_pixel([7000*u.AA, 700*u.nm]), [461.2, 461.2])
+    assert np.allclose(spec.wcs.world_to_pixel(7000*u.AA), [461.2])
 
 def test_create_explicit_fitswcs():
     my_wcs = fitswcs.WCS(header={'CDELT1': 1, 'CRVAL1': 6562.8, 'CUNIT1': 'Angstrom',
@@ -205,6 +279,16 @@ def test_create_with_uncertainty():
 
     assert spec.flux.unit == spec.uncertainty.unit
 
+    # If flux and uncertainty are different sizes then raise exception
+    wavelengths = np.arange(0, 10)
+    flux=100*np.abs(np.random.randn(3, 4, 10))*u.Jy
+    uncertainty = StdDevUncertainty(np.abs(np.random.randn(3, 2, 10))*u.Jy)
+
+    with pytest.raises(ValueError) as e_info:
+        s1d = Spectrum1D(spectral_axis=wavelengths*u.um,
+                         flux=flux,
+                         uncertainty=uncertainty)
+
 
 @remote_access([{'id': '1481190', 'filename': 'L5g_0355+11_Cruz09.fits'}])
 def test_read_linear_solution(remote_data_path):
@@ -234,9 +318,14 @@ def test_repr():
     assert result.startswith('<Spectrum1D(flux=<Quantity [')
     assert 'spectral_axis=<Quantity [' in result
 
-    spec_without_wcs = Spectrum1D(np.random.random(10) * u.Jy)
-    result = repr(spec_without_wcs)
-    assert result == '<Spectrum1D(flux={})>'.format(repr(spec_without_wcs.flux))
+    spec_with_unc = Spectrum1D(spectral_axis=np.linspace(100, 1000, 10) * u.nm,
+                               flux=np.random.random(10) * u.Jy,
+                               uncertainty=StdDevUncertainty(
+                                   np.random.sample(10), unit='Jy'))
+    result = repr(spec_with_unc)
+    assert result.startswith('<Spectrum1D(flux=<Quantity [')
+    assert 'spectral_axis=<Quantity [' in result
+    assert 'uncertainty=StdDevUncertainty(' in result
 
 
 def test_str():
@@ -278,6 +367,10 @@ def test_str():
         assert line.startswith('flux{:2}:'.format(i))
 
     # Test string representation with single-dimensional flux
-    spec_single_flux = Spectrum1D(1 * u.Jy)
+    spec_single_flux = Spectrum1D([1] * u.Jy, [0] * u.nm)
     result = str(spec_single_flux)
-    assert result == 'Spectrum1D (length=1)\nflux:   {}'.format(spec_single_flux.flux)
+    print(spec_single_flux.spectral_axis)
+    assert result == \
+"""Spectrum1D (length=1)
+flux:             [ 1.0 Jy ],  mean=1.0 Jy
+spectral axis:    [ nan nm ],  mean=nan nm"""
